/*@lineinfo:filename=Populate_Cosmos*//*@lineinfo:user-code*//*@lineinfo:1^1*//********************************************************************************/
/*                                                                              */
/*                            Populate_Cosmos.sqlj                              */
/*                            =-=-=-=-=-=-=-=-=-=-                              */
/*                                                                              */
/*                    +------------------------------------+                    */
/*                    | Imports COSMOS information into DB |                    */
/*                    +------------------------------------+                    */
/*                                                                              */
/*                      NCEDC (c)2011  Stephane Zuzlewski                       */
/*                      Berkeley Seismological Laboratory                       */
/*                                                                              */
/********************************************************************************/

import java.lang.reflect.Method;

import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.w3c.dom.Element;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.dom.DOMSource;

import java.sql.*;
import oracle.sqlj.runtime.*;
import sqlj.runtime.*;
import sqlj.runtime.ref.*;
import java.io.File;
import java.io.InputStream;
import java.io.*;
import java.text.*;


/********************************************************************************/
/*      Populate_Cosmos: Class that implements the Update process.                 */
/********************************************************************************/
public class Populate_Cosmos {

	// Main program entry point.
	public static void main(String argv[]) {

		// is there anything to do?
		if (argv.length == 0) {
			printUsage();
			System.exit(1);
		}

		// variables
		String		connectValue = new String ("connect.properties");
		SAXExtract	de=null;
		int		FlagP = 0;
		int		FlagD = 0;

		try {
			// Set the default connection to the URL, user, and password
	 		// specified in the connect.properties file
	 		Oracle.connect(Populate_Cosmos.class, connectValue);
	 		
	 		/*@lineinfo:generated-code*//*@lineinfo:84^5*/

//  ************************************************************
//  #sql { ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY/MM/DD HH24:MI:SS'  };
//  ************************************************************

{
  // declare temps
  oracle.jdbc.OraclePreparedStatement __sJT_st = null;
  sqlj.runtime.ref.DefaultContext __sJT_cc = sqlj.runtime.ref.DefaultContext.getDefaultContext(); if (__sJT_cc==null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext.OracleContext __sJT_ec = ((__sJT_cc.getExecutionContext()==null) ? sqlj.runtime.ExecutionContext.raiseNullExecCtx() : __sJT_cc.getExecutionContext().getOracleContext());
  try {
   __sJT_st = __sJT_ec.prepareOracleStatement(__sJT_cc,"0Populate_Cosmos","ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY/MM/DD HH24:MI:SS'");
   // execute statement
   __sJT_ec.oracleExecuteUpdate();
  } finally { __sJT_ec.oracleClose(); }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:84^72*/
		} catch (SQLException e)
			{ 
			 System.err.println("Error: " + e);
			 System.err.println("Problem while setting NLS_DATE_FORMAT variable.");
			 System.exit(-1);
			}

		// process arguments
		for (int i = 0; i < argv.length; i++) {
			String arg = argv[i];
			if (arg.startsWith("-")) {
				String option = arg.substring(1);
				if (option.equals("h")) {
					printUsage();
					System.exit(1);
				}
				
				if (option.equals("d")) {
					FlagD = 1;
				}
				
				if (option.equals("p")) {
					FlagP = 1;
				}
				
				if (option.equals("f")) {					
					if (++i == argv.length) {
						System.err.println("error: Missing argument to -f option.");
						printUsage();
						System.exit(-1);
					}
								
					de = new SAXExtract (argv[i], FlagD, FlagP);
				}
			}
		}

		try {
			/*@lineinfo:generated-code*//*@lineinfo:123^4*/

//  ************************************************************
//  #sql { COMMIT WORK };
//  ************************************************************

  ((sqlj.runtime.ref.DefaultContext.getDefaultContext().getExecutionContext()==null) ? sqlj.runtime.ExecutionContext.raiseNullExecCtx() : sqlj.runtime.ref.DefaultContext.getDefaultContext().getExecutionContext().getOracleContext()).oracleCommit(sqlj.runtime.ref.DefaultContext.getDefaultContext());


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:123^21*/
		} catch (java.sql.SQLException e) {
			System.err.println("Exception: " + e);
			System.err.println("Problem while commiting changes.");
			System.exit(-1);
		}
	} // main(String[])

    //
    // Private static methods
    //

    /** Prints the usage. */
    private static void printUsage() {
	System.err.println("Import COSMOS information from XML format into Database.");
	System.err.println();
        System.err.println("Usage: java Populate_Cosmos [-h] [-d] [-p] {-f file}");
        System.err.println();

        System.err.println("Options:");
        System.err.println("  -h		Help.");
	System.err.println("  -f file	Specify XML channel map file.");
	System.err.println("  -d		Debugging mode.");
	System.err.println("  -p		Population mode.");
	System.err.println();

    } // printUsage()

}  // class Populate_Cosmos

/********************************************************************************/
/*      SAXExtract: Class that implements the xml extraction.                   */
/********************************************************************************/
class SAXExtract {
	// Constants
	public Document	document = null;
	
	// Feature ids

	/** Namespaces feature id (http://xml.org/sax/features/namespaces). */
	protected static final String NAMESPACES_FEATURE_ID = "http://xml.org/sax/features/namespaces";

	/** Validation feature id (http://xml.org/sax/features/validation). */
	protected static final String VALIDATION_FEATURE_ID = "http://xml.org/sax/features/validation";

	/** Schema validation feature id (http://apache.org/xml/features/validation/schema). */
	protected static final String SCHEMA_VALIDATION_FEATURE_ID = "http://apache.org/xml/features/validation/schema";

	/** Schema full checking feature id (http://apache.org/xml/features/validation/schema-full-checking). */
	protected static final String SCHEMA_FULL_CHECKING_FEATURE_ID = "http://apache.org/xml/features/validation/schema-full-checking";
	
	/** Include ignorable whitespace feature id (http://apache.org/xml/features/dom/include-ignorable-whitespace). */
	protected static final String INCLUDE_IGNORABLE_WHITESPACE_FEATURE_ID = "http://apache.org/xml/features/dom/include-ignorable-whitespace";

	// Default settings

	/** Default parser name (dom.wrappers.Xerces). */
	protected static final String DEFAULT_PARSER_NAME = "dom.wrappers.Xerces";

	/** Default namespaces support (true). */
	protected static final boolean DEFAULT_NAMESPACES = true;

	/** Default validation support (false). */
	protected static final boolean DEFAULT_VALIDATION = false;

	/** Default Schema validation support (false). */
	protected static final boolean DEFAULT_SCHEMA_VALIDATION = false;

	/** Default Schema full checking support (false). */
	protected static final boolean DEFAULT_SCHEMA_FULL_CHECKING = false;
	
	/** Default Ignore WhiteSpace (false). */
	protected static final boolean DEFAULT_IGNORE_WHITESPACE = false;

	/** Number of ignorable whitespace characters. */
	protected long fIgnorableWhitespace;

	/** Document information. */
	protected dom.ParserWrapper.DocumentInfo fDocumentInfo;

	//
	// Public methods
	//

	/** Sets the parser wrapper. */
	public void setDocumentInfo(dom.ParserWrapper.DocumentInfo documentInfo) {
		fDocumentInfo = documentInfo;
	} // setDocumentInfo(ParserWrapper.DocumentInfo)

	
	// Constructor
	public	SAXExtract (String	xml_file, int	FlagD, int	FlagP) {
		dom.ParserWrapper parser = null;
		boolean namespaces = DEFAULT_NAMESPACES;
		boolean validation = DEFAULT_VALIDATION;
		boolean schemaValidation = DEFAULT_SCHEMA_VALIDATION;
		boolean schemaFullChecking = DEFAULT_SCHEMA_FULL_CHECKING;
		boolean	ignoreWhiteSpace = DEFAULT_IGNORE_WHITESPACE;

		// Create parser
		try {
			parser = (dom.ParserWrapper)Class.forName(DEFAULT_PARSER_NAME).newInstance();
		} catch (Exception e) {
			System.err.println("error: Unable to instantiate parser ("+DEFAULT_PARSER_NAME+")");
		}
		
		// set parser features
		try {
			parser.setFeature(NAMESPACES_FEATURE_ID, namespaces);
		} catch (SAXException e) {
			System.err.println("warning: Parser does not support feature ("+NAMESPACES_FEATURE_ID+")");
		}
		
		try {
			parser.setFeature(VALIDATION_FEATURE_ID, validation);
		} catch (SAXException e) {
			System.err.println("warning: Parser does not support feature ("+VALIDATION_FEATURE_ID+")");
		}
		
		try {
			parser.setFeature(SCHEMA_VALIDATION_FEATURE_ID, schemaValidation);
		} catch (SAXException e) {
			System.err.println("warning: Parser does not support feature ("+SCHEMA_VALIDATION_FEATURE_ID
+")");
		}
		
		try {
			parser.setFeature(SCHEMA_FULL_CHECKING_FEATURE_ID, schemaFullChecking);
		} catch (SAXException e) {
			System.err.println("warning: Parser does not support feature ("+SCHEMA_FULL_CHECKING_FEATURE_ID+")");
		}

		try {
			parser.setFeature(INCLUDE_IGNORABLE_WHITESPACE_FEATURE_ID, ignoreWhiteSpace);
		} catch (SAXException e) {
			System.err.println("warning: Parser does not support feature ("+INCLUDE_IGNORABLE_WHITESPACE_FEATURE_ID+")");
		}

		// Parse XML file
		try {
			document = parser.parse(xml_file);
		} catch (java.lang.Exception e) {
			// ignore
		}
		
		// Channel_Mapping Section
		Node child = document.getFirstChild().getFirstChild();

		dom.ParserWrapper.DocumentInfo documentInfo = parser.getDocumentInfo();
		this.setDocumentInfo(documentInfo);
		
		while (child != null) {
			int type = child.getNodeType();

			switch (type) {
				case Node.ELEMENT_NODE: {
					if (child.getNodeName().equals(new String ("channel")) == true) {
						if (FlagD == 1) {
							System.out.println ("channel");
						}
						
						this.SAXChannelExtract (child, FlagD, FlagP);
					}
					else	{
						System.out.println ("Warning: Found " + child.getNodeName().trim() + " Node");
					}
				}
			}

		 child = child.getNextSibling();
		}
	}
	
	/** Extract channel information */
	public	void	SAXChannelExtract (Node	node, int	FlagD, int	FlagP) {
		// Channel_Mapping Section
		NamedNodeMap	attrs = node.getAttributes();
		String		ondate = attrs.getNamedItem("begin").getNodeValue().trim();
		String		offdate = new String (attrs.getNamedItem("end").getNodeValue().trim());
		int		data_nb = 0;
		int		data_pchannel_nb = 0;
		int		digi_nb = 0;
		int		digi_pchannel_nb = 0;
		int		filamp_nb = 0;
		int		filamp_pchannel_nb = 0;
		String		sensor_name = new String("Unknown");
		String		data_name = new String("Unknown");
		int		netid = 199;
		int		orntid = 0;
		int		dip = 0;
		int		azimuth = 0;

		
		if (FlagD == 1) {
			System.out.println ("\tondate: " + ondate + " | offdate: " + offdate);
		}

		// Network Section
		Node _cs_network = node.getFirstChild();
		String cs_network = new String(_cs_network.getFirstChild().getNodeValue());

		// Station Section
		Node _cs_station = _cs_network.getNextSibling();
		String cs_station = new String(_cs_station.getFirstChild().getNodeValue());

		// Channel Section
		Node _cs_channel = _cs_station.getNextSibling();
		String cs_channel = new String(_cs_channel.getFirstChild().getNodeValue());

		// Location Section
		Node _cs_location = _cs_channel.getNextSibling();
		String cs_location = new String(_cs_location.getFirstChild().getNodeValue());

		// Site Code Section
		Node _cs_code = _cs_location.getNextSibling();
		String cs_code = new String(_cs_code.getFirstChild().getNodeValue());

		if (FlagD == 1) {
			System.out.println("\tsite code: \t" + cs_network + " " + cs_station + " " + cs_channel +
						" " + cs_location + " ==> " + cs_code);
		}

		if (cs_location.equals("--")) {
			cs_location = new String("  ");
		}

		// Retrieve datalogger information
		try {
			/*@lineinfo:generated-code*//*@lineinfo:351^4*/

//  ************************************************************
//  #sql { SELECT sdlc.data_nb, sdlc.pchannel_nb, sdpc.digi_nb, sdpc.pchannel_nb
//  			      
//                                FROM station_datalogger_lchannel sdlc, station_digitizer_pchannel sdpc
//                                WHERE sdlc.net = :cs_network AND sdlc.sta = :cs_station AND
//                                   sdlc.seedchan = :cs_channel AND sdlc.location = :cs_location AND
//                                   :ondate >= sdlc.ondate AND :ondate < sdlc.offdate AND
//                                   sdlc.sta = sdpc.sta AND sdlc.net = sdpc.net AND
//                                   sdlc.ondate = sdpc.ondate AND
//                                   sdpc.data_nb = sdlc.data_nb AND
//                                   sdpc.data_pchannel = sdlc.pchannel_nb };
//  ************************************************************

{
  // declare temps
  oracle.jdbc.OraclePreparedStatement __sJT_st = null;
  sqlj.runtime.ref.DefaultContext __sJT_cc = sqlj.runtime.ref.DefaultContext.getDefaultContext(); if (__sJT_cc==null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext.OracleContext __sJT_ec = ((__sJT_cc.getExecutionContext()==null) ? sqlj.runtime.ExecutionContext.raiseNullExecCtx() : __sJT_cc.getExecutionContext().getOracleContext());
  oracle.jdbc.OracleResultSet __sJT_rs = null;
  try {
   __sJT_st = __sJT_ec.prepareOracleStatement(__sJT_cc,"1Populate_Cosmos","SELECT sdlc.data_nb, sdlc.pchannel_nb, sdpc.digi_nb, sdpc.pchannel_nb\n\t\t\t       \n                              FROM station_datalogger_lchannel sdlc, station_digitizer_pchannel sdpc\n                              WHERE sdlc.net =  :1  AND sdlc.sta =  :2  AND\n                                 sdlc.seedchan =  :3  AND sdlc.location =  :4  AND\n                                  :5  >= sdlc.ondate AND  :6  < sdlc.offdate AND\n                                 sdlc.sta = sdpc.sta AND sdlc.net = sdpc.net AND\n                                 sdlc.ondate = sdpc.ondate AND\n                                 sdpc.data_nb = sdlc.data_nb AND\n                                 sdpc.data_pchannel = sdlc.pchannel_nb");
   if (__sJT_ec.isNew())
   {
     __sJT_st.setFetchSize(2);
   }
   // set IN parameters
   __sJT_st.setString(1,cs_network);
   __sJT_st.setString(2,cs_station);
   __sJT_st.setString(3,cs_channel);
   __sJT_st.setString(4,cs_location);
   __sJT_st.setString(5,ondate);
   __sJT_st.setString(6,ondate);
   // execute query
   __sJT_rs = __sJT_ec.oracleExecuteQuery();
   if (__sJT_rs.getMetaData().getColumnCount() != 4) sqlj.runtime.error.RuntimeRefErrors.raise_WRONG_NUM_COLS(4,__sJT_rs.getMetaData().getColumnCount());
   if (!__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
   // retrieve OUT parameters
   data_nb = __sJT_rs.getInt(1); if (__sJT_rs.wasNull()) throw new sqlj.runtime.SQLNullException();
   data_pchannel_nb = __sJT_rs.getInt(2); if (__sJT_rs.wasNull()) throw new sqlj.runtime.SQLNullException();
   digi_nb = __sJT_rs.getInt(3); if (__sJT_rs.wasNull()) throw new sqlj.runtime.SQLNullException();
   digi_pchannel_nb = __sJT_rs.getInt(4); if (__sJT_rs.wasNull()) throw new sqlj.runtime.SQLNullException();
   if (__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
  } finally { if (__sJT_rs!=null) __sJT_rs.close(); __sJT_ec.oracleClose(); }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:360^71*/
		} catch (java.sql.SQLException e) {
			// System.err.println("Exception: " + e);
			// System.err.println("Problem while retrieving datalogger information.");
			// System.exit(-1);
			System.out.println ("WARNING: Could not find datalogger information for " + cs_network + " " + cs_station + " " + cs_channel + " " + cs_location + " " + ondate);
		}

		// Retrieve datalogger name
		try {
			/*@lineinfo:generated-code*//*@lineinfo:370^4*/

//  ************************************************************
//  #sql { SELECT d.data_type
//  			      
//  			      FROM datalogger d, station_datalogger sd
//  			      WHERE d.data_id = sd.data_id AND
//  				 sd.net = :cs_network AND sd.sta = :cs_station AND
//  				 :ondate >= sd.ondate AND :ondate < sd.offdate AND
//  				 sd.data_nb = :data_nb };
//  ************************************************************

{
  // declare temps
  oracle.jdbc.OraclePreparedStatement __sJT_st = null;
  sqlj.runtime.ref.DefaultContext __sJT_cc = sqlj.runtime.ref.DefaultContext.getDefaultContext(); if (__sJT_cc==null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext.OracleContext __sJT_ec = ((__sJT_cc.getExecutionContext()==null) ? sqlj.runtime.ExecutionContext.raiseNullExecCtx() : __sJT_cc.getExecutionContext().getOracleContext());
  oracle.jdbc.OracleResultSet __sJT_rs = null;
  try {
   __sJT_st = __sJT_ec.prepareOracleStatement(__sJT_cc,"2Populate_Cosmos","SELECT d.data_type\n\t\t\t       \n\t\t\t      FROM datalogger d, station_datalogger sd\n\t\t\t      WHERE d.data_id = sd.data_id AND\n\t\t\t\t sd.net =  :1  AND sd.sta =  :2  AND\n\t\t\t\t  :3  >= sd.ondate AND  :4  < sd.offdate AND\n\t\t\t\t sd.data_nb =  :5");
   if (__sJT_ec.isNew())
   {
     __sJT_st.setFetchSize(2);
   }
   // set IN parameters
   __sJT_st.setString(1,cs_network);
   __sJT_st.setString(2,cs_station);
   __sJT_st.setString(3,ondate);
   __sJT_st.setString(4,ondate);
   __sJT_st.setInt(5,data_nb);
   // execute query
   __sJT_rs = __sJT_ec.oracleExecuteQuery();
   if (__sJT_rs.getMetaData().getColumnCount() != 1) sqlj.runtime.error.RuntimeRefErrors.raise_WRONG_NUM_COLS(1,__sJT_rs.getMetaData().getColumnCount());
   if (!__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
   // retrieve OUT parameters
   data_name = __sJT_rs.getString(1);
   if (__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
  } finally { if (__sJT_rs!=null) __sJT_rs.close(); __sJT_ec.oracleClose(); }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:376^27*/
		} catch (java.sql.SQLException e) {
			// System.err.println("Exception: " + e);
			// System.err.println("Problem while retrieving datalogger name.");
			// System.exit(-1);
			System.out.println ("WARNING: Could not find datalogger name for " + cs_network + " " + cs_station + " " + cs_channel + " " + cs_location + " " + ondate);
		}

		// Filter-Amplifier?
		try {
			/*@lineinfo:generated-code*//*@lineinfo:386^4*/

//  ************************************************************
//  #sql { SELECT filamp_nb, pchannel_nb
//  			      
//  			      FROM station_filamp_pchannel
//                                WHERE net = :cs_network AND sta = :cs_station AND
//                                   :ondate >= ondate AND :ondate < offdate AND next_hard_type = 'D' AND
//                                   next_hard_nb = :digi_nb AND
//                                   next_hard_pchannel = :digi_pchannel_nb };
//  ************************************************************

{
  // declare temps
  oracle.jdbc.OraclePreparedStatement __sJT_st = null;
  sqlj.runtime.ref.DefaultContext __sJT_cc = sqlj.runtime.ref.DefaultContext.getDefaultContext(); if (__sJT_cc==null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext.OracleContext __sJT_ec = ((__sJT_cc.getExecutionContext()==null) ? sqlj.runtime.ExecutionContext.raiseNullExecCtx() : __sJT_cc.getExecutionContext().getOracleContext());
  oracle.jdbc.OracleResultSet __sJT_rs = null;
  try {
   __sJT_st = __sJT_ec.prepareOracleStatement(__sJT_cc,"3Populate_Cosmos","SELECT filamp_nb, pchannel_nb\n\t\t\t       \n\t\t\t      FROM station_filamp_pchannel\n                              WHERE net =  :1  AND sta =  :2  AND\n                                  :3  >= ondate AND  :4  < offdate AND next_hard_type = 'D' AND\n                                 next_hard_nb =  :5  AND\n                                 next_hard_pchannel =  :6");
   if (__sJT_ec.isNew())
   {
     __sJT_st.setFetchSize(2);
   }
   // set IN parameters
   __sJT_st.setString(1,cs_network);
   __sJT_st.setString(2,cs_station);
   __sJT_st.setString(3,ondate);
   __sJT_st.setString(4,ondate);
   __sJT_st.setInt(5,digi_nb);
   __sJT_st.setInt(6,digi_pchannel_nb);
   // execute query
   __sJT_rs = __sJT_ec.oracleExecuteQuery();
   if (__sJT_rs.getMetaData().getColumnCount() != 2) sqlj.runtime.error.RuntimeRefErrors.raise_WRONG_NUM_COLS(2,__sJT_rs.getMetaData().getColumnCount());
   if (!__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
   // retrieve OUT parameters
   filamp_nb = __sJT_rs.getInt(1); if (__sJT_rs.wasNull()) throw new sqlj.runtime.SQLNullException();
   filamp_pchannel_nb = __sJT_rs.getInt(2); if (__sJT_rs.wasNull()) throw new sqlj.runtime.SQLNullException();
   if (__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
  } finally { if (__sJT_rs!=null) __sJT_rs.close(); __sJT_ec.oracleClose(); }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:392^72*/
		} catch (java.sql.SQLException e) {
			// System.err.println("Exception: " + e);
			// System.err.println("Problem while retrieving filter-amplifier information.");
			// System.exit(-1);
		}

		// Retrieve sensor name
		try {
			if (filamp_nb == 0) {
				/*@lineinfo:generated-code*//*@lineinfo:402^5*/

//  ************************************************************
//  #sql { SELECT s.name, ssc.dip, ssc.azimuth
//  				      
//                               	      FROM sensor s, station_sensor ss, station_sensor_component ssc
//                               	      WHERE s.sensor_id = ss.sensor_id AND ss.net = :cs_network AND ss.sta = :cs_station AND
//                                     	 :ondate >= ss.ondate AND :ondate < ss.offdate AND ss.sensor_nb = ssc.sensor_nb AND
//                                     	 ss.sta = ssc.sta AND ss.net = ssc.net AND
//                                     	 ss.ondate = ssc.ondate AND
//  				   	 ssc.next_hard_type = 'D' AND
//                                     	 ssc.next_hard_nb = :digi_nb AND
//                                     	 ssc.next_hard_pchannel = :digi_pchannel_nb };
//  ************************************************************

{
  // declare temps
  oracle.jdbc.OraclePreparedStatement __sJT_st = null;
  sqlj.runtime.ref.DefaultContext __sJT_cc = sqlj.runtime.ref.DefaultContext.getDefaultContext(); if (__sJT_cc==null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext.OracleContext __sJT_ec = ((__sJT_cc.getExecutionContext()==null) ? sqlj.runtime.ExecutionContext.raiseNullExecCtx() : __sJT_cc.getExecutionContext().getOracleContext());
  oracle.jdbc.OracleResultSet __sJT_rs = null;
  try {
   __sJT_st = __sJT_ec.prepareOracleStatement(__sJT_cc,"4Populate_Cosmos","SELECT s.name, ssc.dip, ssc.azimuth\n\t\t\t\t       \n                             \t      FROM sensor s, station_sensor ss, station_sensor_component ssc\n                             \t      WHERE s.sensor_id = ss.sensor_id AND ss.net =  :1  AND ss.sta =  :2  AND\n                                   \t  :3  >= ss.ondate AND  :4  < ss.offdate AND ss.sensor_nb = ssc.sensor_nb AND\n                                   \t ss.sta = ssc.sta AND ss.net = ssc.net AND\n                                   \t ss.ondate = ssc.ondate AND\n\t\t\t\t   \t ssc.next_hard_type = 'D' AND\n                                   \t ssc.next_hard_nb =  :5  AND\n                                   \t ssc.next_hard_pchannel =  :6");
   if (__sJT_ec.isNew())
   {
     __sJT_st.setFetchSize(2);
   }
   // set IN parameters
   __sJT_st.setString(1,cs_network);
   __sJT_st.setString(2,cs_station);
   __sJT_st.setString(3,ondate);
   __sJT_st.setString(4,ondate);
   __sJT_st.setInt(5,digi_nb);
   __sJT_st.setInt(6,digi_pchannel_nb);
   // execute query
   __sJT_rs = __sJT_ec.oracleExecuteQuery();
   if (__sJT_rs.getMetaData().getColumnCount() != 3) sqlj.runtime.error.RuntimeRefErrors.raise_WRONG_NUM_COLS(3,__sJT_rs.getMetaData().getColumnCount());
   if (!__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
   // retrieve OUT parameters
   sensor_name = __sJT_rs.getString(1);
   dip = __sJT_rs.getInt(2); if (__sJT_rs.wasNull()) throw new sqlj.runtime.SQLNullException();
   azimuth = __sJT_rs.getInt(3); if (__sJT_rs.wasNull()) throw new sqlj.runtime.SQLNullException();
   if (__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
  } finally { if (__sJT_rs!=null) __sJT_rs.close(); __sJT_ec.oracleClose(); }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:411^80*/
			} else {
				/*@lineinfo:generated-code*//*@lineinfo:413^5*/

//  ************************************************************
//  #sql { SELECT s.name, ssc.dip, ssc.azimuth
//  				      
//                               	      FROM sensor s, station_sensor ss, station_sensor_component ssc
//                               	      WHERE s.sensor_id = ss.sensor_id AND ss.net = :cs_network AND ss.sta = :cs_station AND
//                                      	 :ondate >= ss.ondate AND :ondate < ss.offdate AND ss.sensor_nb = ssc.sensor_nb AND
//                                      	 ss.sta = ssc.sta AND ss.net = ssc.net AND
//                                      	 ss.ondate = ssc.ondate AND
//  				    	 ssc.next_hard_type = 'F' AND
//                                      	 ssc.next_hard_nb = :filamp_nb AND
//                                      	 ssc.next_hard_pchannel = :filamp_pchannel_nb };
//  ************************************************************

{
  // declare temps
  oracle.jdbc.OraclePreparedStatement __sJT_st = null;
  sqlj.runtime.ref.DefaultContext __sJT_cc = sqlj.runtime.ref.DefaultContext.getDefaultContext(); if (__sJT_cc==null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext.OracleContext __sJT_ec = ((__sJT_cc.getExecutionContext()==null) ? sqlj.runtime.ExecutionContext.raiseNullExecCtx() : __sJT_cc.getExecutionContext().getOracleContext());
  oracle.jdbc.OracleResultSet __sJT_rs = null;
  try {
   __sJT_st = __sJT_ec.prepareOracleStatement(__sJT_cc,"5Populate_Cosmos","SELECT s.name, ssc.dip, ssc.azimuth\n\t\t\t\t       \n                             \t      FROM sensor s, station_sensor ss, station_sensor_component ssc\n                             \t      WHERE s.sensor_id = ss.sensor_id AND ss.net =  :1  AND ss.sta =  :2  AND\n                                    \t  :3  >= ss.ondate AND  :4  < ss.offdate AND ss.sensor_nb = ssc.sensor_nb AND\n                                    \t ss.sta = ssc.sta AND ss.net = ssc.net AND\n                                    \t ss.ondate = ssc.ondate AND\n\t\t\t\t    \t ssc.next_hard_type = 'F' AND\n                                    \t ssc.next_hard_nb =  :5  AND\n                                    \t ssc.next_hard_pchannel =  :6");
   if (__sJT_ec.isNew())
   {
     __sJT_st.setFetchSize(2);
   }
   // set IN parameters
   __sJT_st.setString(1,cs_network);
   __sJT_st.setString(2,cs_station);
   __sJT_st.setString(3,ondate);
   __sJT_st.setString(4,ondate);
   __sJT_st.setInt(5,filamp_nb);
   __sJT_st.setInt(6,filamp_pchannel_nb);
   // execute query
   __sJT_rs = __sJT_ec.oracleExecuteQuery();
   if (__sJT_rs.getMetaData().getColumnCount() != 3) sqlj.runtime.error.RuntimeRefErrors.raise_WRONG_NUM_COLS(3,__sJT_rs.getMetaData().getColumnCount());
   if (!__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
   // retrieve OUT parameters
   sensor_name = __sJT_rs.getString(1);
   dip = __sJT_rs.getInt(2); if (__sJT_rs.wasNull()) throw new sqlj.runtime.SQLNullException();
   azimuth = __sJT_rs.getInt(3); if (__sJT_rs.wasNull()) throw new sqlj.runtime.SQLNullException();
   if (__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
  } finally { if (__sJT_rs!=null) __sJT_rs.close(); __sJT_ec.oracleClose(); }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:422^83*/
			}
		} catch (java.sql.SQLException e) {
			// System.err.println("Exception: " + e);
			// System.err.println("Problem while retrieving sensor name.");
			// System.exit(-1);
			System.out.println ("WARNING: Could not find sensor name for " + cs_network + " " + cs_station + " " + cs_channel + " " + cs_location + " " + ondate);
		}

		// Remove extra information in sensor's name
		if (sensor_name.indexOf("(") > 0) {
			sensor_name = sensor_name.substring(0,sensor_name.indexOf("(")-1);
		}

		// Retrieve network Id
		try {
			/*@lineinfo:generated-code*//*@lineinfo:438^4*/

//  ************************************************************
//  #sql { SELECT netid
//  			      
//  			      FROM C_Net
//  			      WHERE net = :cs_network };
//  ************************************************************

{
  // declare temps
  oracle.jdbc.OraclePreparedStatement __sJT_st = null;
  sqlj.runtime.ref.DefaultContext __sJT_cc = sqlj.runtime.ref.DefaultContext.getDefaultContext(); if (__sJT_cc==null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext.OracleContext __sJT_ec = ((__sJT_cc.getExecutionContext()==null) ? sqlj.runtime.ExecutionContext.raiseNullExecCtx() : __sJT_cc.getExecutionContext().getOracleContext());
  oracle.jdbc.OracleResultSet __sJT_rs = null;
  try {
   __sJT_st = __sJT_ec.prepareOracleStatement(__sJT_cc,"6Populate_Cosmos","SELECT netid\n\t\t\t       \n\t\t\t      FROM C_Net\n\t\t\t      WHERE net =  :1");
   if (__sJT_ec.isNew())
   {
     __sJT_st.setFetchSize(2);
   }
   // set IN parameters
   __sJT_st.setString(1,cs_network);
   // execute query
   __sJT_rs = __sJT_ec.oracleExecuteQuery();
   if (__sJT_rs.getMetaData().getColumnCount() != 1) sqlj.runtime.error.RuntimeRefErrors.raise_WRONG_NUM_COLS(1,__sJT_rs.getMetaData().getColumnCount());
   if (!__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
   // retrieve OUT parameters
   netid = __sJT_rs.getInt(1); if (__sJT_rs.wasNull()) throw new sqlj.runtime.SQLNullException();
   if (__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
  } finally { if (__sJT_rs!=null) __sJT_rs.close(); __sJT_ec.oracleClose(); }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:441^33*/
		} catch (java.sql.SQLException e) {
			// System.err.println("Exception: " + e);
			// System.err.println("Problem while retrieving network Id.");
			// System.exit(-1);
			System.out.println ("WARNING: Could not find network Id for " + cs_network);
		}

		// Compute orientation Id
		 if (dip == 0) {
			if (azimuth == 0)
				orntid = 360;
			else	orntid = azimuth;
		} else {
			if (dip == -90)
				orntid = 400;
			else	orntid = 401;
 		}

		if (FlagD == 1) {
			System.out.println("\tsensor: \t" + sensor_name);
			System.out.println("\tdatalogger: \t" + data_name);
			System.out.println("\tnetwork id: \t" + netid);
			System.out.println("\torient. id: \t" + orntid);
		}

		if (FlagP == 1) {
			if (cs_location.equals("--")) {
				cs_location = "  ";
			}

			// Populate channel in database
			this.SAXChannelInsert ( cs_network, cs_station, cs_channel, cs_location, netid, orntid,
						cs_code, sensor_name, data_name, ondate, offdate, FlagD );
		}
	}


	/** Populates channel information */
	public	void	SAXChannelInsert (String net, String sta, String cha, String loc, int netid, int orntid, String cs_code,
					  String sname, String dname, String ondate, String offdate, int FlagD) {
		int	nbcount = 0;


		// Is site already in database?
		try {
			/*@lineinfo:generated-code*//*@lineinfo:487^4*/

//  ************************************************************
//  #sql { SELECT COUNT(*)  FROM C_ChannelData WHERE net = :net AND sta = :sta AND seedchan = :cha AND location = :loc AND siteid = :cs_code AND snsname = :sname AND lgrname = :dname AND ondate = :ondate AND offdate = :offdate };
//  ************************************************************

{
  // declare temps
  oracle.jdbc.OraclePreparedStatement __sJT_st = null;
  sqlj.runtime.ref.DefaultContext __sJT_cc = sqlj.runtime.ref.DefaultContext.getDefaultContext(); if (__sJT_cc==null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext.OracleContext __sJT_ec = ((__sJT_cc.getExecutionContext()==null) ? sqlj.runtime.ExecutionContext.raiseNullExecCtx() : __sJT_cc.getExecutionContext().getOracleContext());
  oracle.jdbc.OracleResultSet __sJT_rs = null;
  try {
   __sJT_st = __sJT_ec.prepareOracleStatement(__sJT_cc,"7Populate_Cosmos","SELECT COUNT(*)   FROM C_ChannelData WHERE net =  :1  AND sta =  :2  AND seedchan =  :3  AND location =  :4  AND siteid =  :5  AND snsname =  :6  AND lgrname =  :7  AND ondate =  :8  AND offdate =  :9");
   if (__sJT_ec.isNew())
   {
     __sJT_st.setFetchSize(2);
   }
   // set IN parameters
   __sJT_st.setString(1,net);
   __sJT_st.setString(2,sta);
   __sJT_st.setString(3,cha);
   __sJT_st.setString(4,loc);
   __sJT_st.setString(5,cs_code);
   __sJT_st.setString(6,sname);
   __sJT_st.setString(7,dname);
   __sJT_st.setString(8,ondate);
   __sJT_st.setString(9,offdate);
   // execute query
   __sJT_rs = __sJT_ec.oracleExecuteQuery();
   if (__sJT_rs.getMetaData().getColumnCount() != 1) sqlj.runtime.error.RuntimeRefErrors.raise_WRONG_NUM_COLS(1,__sJT_rs.getMetaData().getColumnCount());
   if (!__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
   // retrieve OUT parameters
   nbcount = __sJT_rs.getInt(1); if (__sJT_rs.wasNull()) throw new sqlj.runtime.SQLNullException();
   if (__sJT_rs.next()) sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
  } finally { if (__sJT_rs!=null) __sJT_rs.close(); __sJT_ec.oracleClose(); }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:487^238*/
		} catch (java.sql.SQLException e) {
			System.err.println("Exception: " + e);
			System.err.println("Problem while selecting from C_ChannelData table.");
			System.exit(-1);
		}

		if (nbcount == 0) {
			if (FlagD == 1) {
				System.out.println ("\tPopulating channel information into database");
			}

			// Delete entries from the database
			this.SAXChannelDelete (net, sta, cha, loc, ondate, offdate);

			// Populate channel mapping into database
			try {
				/*@lineinfo:generated-code*//*@lineinfo:504^5*/

//  ************************************************************
//  #sql { INSERT INTO C_ChannelData (net, sta, seedchan, location, ondate, offdate, netid, lgrname, snsname, siteid, orntid, timeid, datumid, stavs30, stageoid, lddate) VALUES(:net, :sta, :cha, :loc, :ondate, :offdate, :netid, :dname, :sname, :cs_code, :orntid, 5, 1, NULL, NULL, SYSDATE) };
//  ************************************************************

{
  // declare temps
  oracle.jdbc.OraclePreparedStatement __sJT_st = null;
  sqlj.runtime.ref.DefaultContext __sJT_cc = sqlj.runtime.ref.DefaultContext.getDefaultContext(); if (__sJT_cc==null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext.OracleContext __sJT_ec = ((__sJT_cc.getExecutionContext()==null) ? sqlj.runtime.ExecutionContext.raiseNullExecCtx() : __sJT_cc.getExecutionContext().getOracleContext());
   __sJT_st = __sJT_ec.prepareOracleBatchableStatement(__sJT_cc,"8Populate_Cosmos","INSERT INTO C_ChannelData (net, sta, seedchan, location, ondate, offdate, netid, lgrname, snsname, siteid, orntid, timeid, datumid, stavs30, stageoid, lddate) VALUES( :1 ,  :2 ,  :3 ,  :4 ,  :5 ,  :6 ,  :7 ,  :8 ,  :9 ,  :10 ,  :11 , 5, 1, NULL, NULL, SYSDATE)");
   // set IN parameters
   __sJT_st.setString(1,net);
   __sJT_st.setString(2,sta);
   __sJT_st.setString(3,cha);
   __sJT_st.setString(4,loc);
   __sJT_st.setString(5,ondate);
   __sJT_st.setString(6,offdate);
   __sJT_st.setInt(7,netid);
   __sJT_st.setString(8,dname);
   __sJT_st.setString(9,sname);
   __sJT_st.setString(10,cs_code);
   __sJT_st.setInt(11,orntid);
   // execute statement
   __sJT_ec.oracleExecuteBatchableUpdate();
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:504^289*/
			} catch (java.sql.SQLException e) {
				System.err.println("Exception: " + e);
				System.err.println("Problem while inserting into C_ChannelData table.");
				System.exit(-1);
			}
		}
		else if (nbcount == 1) {
			if (FlagD == 1) {
				System.out.println ("\tChannel information already in database");
			}
		}
		else	{
			System.out.println ("\tWarning: Duplicate channel information for " + net + "." + sta + "." + cha + "." + loc);
		}
	}
		
	/** Delete channel information for a specified time span */
	public	void	SAXChannelDelete (String net, String sta, String cha, String loc, String ondate, String offdate) {
		try {
			/*@lineinfo:generated-code*//*@lineinfo:524^4*/

//  ************************************************************
//  #sql { DELETE FROM C_ChannelData WHERE net = :net AND sta = :sta AND seedchan = :cha AND location = :loc AND (ondate < :offdate AND offdate > :ondate) };
//  ************************************************************

{
  // declare temps
  oracle.jdbc.OraclePreparedStatement __sJT_st = null;
  sqlj.runtime.ref.DefaultContext __sJT_cc = sqlj.runtime.ref.DefaultContext.getDefaultContext(); if (__sJT_cc==null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext.OracleContext __sJT_ec = ((__sJT_cc.getExecutionContext()==null) ? sqlj.runtime.ExecutionContext.raiseNullExecCtx() : __sJT_cc.getExecutionContext().getOracleContext());
   __sJT_st = __sJT_ec.prepareOracleBatchableStatement(__sJT_cc,"9Populate_Cosmos","DELETE FROM C_ChannelData WHERE net =  :1  AND sta =  :2  AND seedchan =  :3  AND location =  :4  AND (ondate <  :5  AND offdate >  :6 )");
   // set IN parameters
   __sJT_st.setString(1,net);
   __sJT_st.setString(2,sta);
   __sJT_st.setString(3,cha);
   __sJT_st.setString(4,loc);
   __sJT_st.setString(5,offdate);
   __sJT_st.setString(6,ondate);
   // execute statement
   __sJT_ec.oracleExecuteBatchableUpdate();
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:524^153*/
		} catch (java.sql.SQLException e) {
			System.err.println("Exception: " + e);
			System.err.println("Problem while deleting from C_ChannelData table.");
			System.exit(-1);
		}	
	}
}  // Class SAXExtract


/********************************************************************************/
/*      Constant: Class that implements constants.                              */
/********************************************************************************/
class Constant {
        public final static String VERSION = "1.0";
}/*@lineinfo:generated-code*/