#! /usr/local/bin/perl

require 5.002;
use strict;
use FileHandle;
use Getopt::Std;
use File::Basename;
use DBI;

use vars qw ( $VERSION $cmdname $opt_h $opt_f );

$VERSION = "1.0 (2011/10/13)";

########################################################################
#	print_syntax	- print syntax and exit.
########################################################################
sub print_syntax {
    my ($cmdname) = @_;
    printf	
"$cmdname version $VERSION
$cmdname - Converts a text file (S N C L SiteCode) to XML format.
Syntax: $cmdname  [-h] -f txtFile
where:
	-f txtFile     Specifies the log file for valid requests.
	-h             Help - prints this help message.
";
    exit(0);
}


exit &main();

sub main
{

####################################################################
# Initialization.
####################################################################

    STDERR->autoflush(1);
    STDOUT->autoflush(1);

########################################################################
#	Parse command line.
########################################################################
    $cmdname = basename($0);
    &getopts ('hf:');
    &print_syntax($cmdname) if ($opt_h);

    my($txtfile, $txt);
    $txtfile = $opt_f;
    die ("Error: Missing txtfile\n")  if ($txtfile =~ /^\s*$/);
    undef @ARGV;
    open ($txt, "$txtfile") || die ("Error: unable to open $txtfile\n");

    my ($dbh);
    unless ( $dbh = DBI->connect("DBI:Oracle:","","",{AutoCommit => 0}) ) {
	die ("Error: unable to connect to Oracle\n");
    }

    print("<cosmos>\n");
    my(@a);
    while (<$txt>) { 
	# print ($_);
	chomp;
	@a = split;

	my ($dbloc,$sql_statement,$cursor,$ondate,$offdate);
	if ($a[3] eq "--") {
		$dbloc = "  ";
	}
	else {
		$dbloc = $a[3];
	}

	$sql_statement = "SELECT DISTINCT ondate, offdate FROM Station_Datalogger_LChannel WHERE net='$a[1]' AND sta='$a[0]' AND seedchan='$a[2]' AND location='$dbloc' ORDER BY ondate,offdate";
	unless ($cursor = $dbh->prepare("$sql_statement")) {
		print ("Error: unable to prepare cursor\n");
		$dbh->disconnect;
		exit;
	}

	unless ($cursor->execute) {
		print ("Error: unable to execute cursor\n");
                $dbh->disconnect;
                exit;
        }
	
	while (($ondate,$offdate) = $cursor->fetchrow_array) {
		print ("<channel begin=\"$ondate\" end=\"$offdate\">\n");
		print ("<network>$a[1]</network>\n");
		print ("<station>$a[0]</station>\n");
		print ("<channel>$a[2]</channel>\n");
		print ("<location>$a[3]</location>\n");
		print ("<site_code>$a[4]</site_code>\n");
		print ("</channel>\n");
	}

	unless ($cursor->finish) {
		print ("Error: unable to finish cursor\n");
                $dbh->disconnect;
                exit;
        }
    }	
    close ($txt);
    $dbh->disconnect;
    print("</cosmos>\n");
    exit(0);
}
